// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace ScreenRecorder::Constants {

const char TOOLSSETTINGSPAGE_ID[] = "Z.ScreenRecorder";
const char LOGGING_CATEGORY[] = "qtc.screenrecorder";
const char ACTION_ID[] = "ScreenRecorder.Action";
const char FFMPEG_COMMAND[] = "ffmpeg";
const char FFPROBE_COMMAND[] = "ffprobe";
const char FFMPEG_DOWNLOAD_URL[] = "https://ffmpeg.org/download.html";

} // namespace ScreenRecorder::Constants
