// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QLatin1String>

namespace QmlDesigner::Constants {

constexpr QLatin1String oldQuick3dAssetsFolder{"Quick3DAssets"};
constexpr QLatin1String quick3DComponentsFolder{"QtQuick3D"};

} // namespace QmlDesigner::Constants
