// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace EmacsKeys {
namespace Constants {

const char DELETE_CHARACTER[]         = "EmacsKeys.DeleteCharacter";
const char KILL_WORD[]                = "EmacsKeys.KillWord";
const char KILL_LINE[]                = "EmacsKeys.KillLine";
const char INSERT_LINE_AND_INDENT[]   = "EmacsKeys.InsertLineAndIndent";

const char GOTO_FILE_START[]          = "EmacsKeys.GotoFileStart";
const char GOTO_FILE_END[]            = "EmacsKeys.GotoFileEnd";
const char GOTO_LINE_START[]          = "EmacsKeys.GotoLineStart";
const char GOTO_LINE_END[]            = "EmacsKeys.GotoLineEnd";
const char GOTO_NEXT_LINE[]           = "EmacsKeys.GotoNextLine";
const char GOTO_PREVIOUS_LINE[]       = "EmacsKeys.GotoPreviousLine";
const char GOTO_NEXT_CHARACTER[]      = "EmacsKeys.GotoNextCharacter";
const char GOTO_PREVIOUS_CHARACTER[]  = "EmacsKeys.GotoPreviousCharacter";
const char GOTO_NEXT_WORD[]           = "EmacsKeys.GotoNextWord";
const char GOTO_PREVIOUS_WORD[]       = "EmacsKeys.GotoPreviousWord";

const char MARK[]                     = "EmacsKeys.Mark";
const char EXCHANGE_CURSOR_AND_MARK[] = "EmacsKeys.ExchangeCursorAndMark";
const char COPY[]                     = "EmacsKeys.Copy";
const char CUT[]                      = "EmacsKeys.Cut";
const char YANK[]                     = "EmacsKeys.Yank";

const char SCROLL_HALF_DOWN[]         = "EmacsKeys.ScrollHalfDown";
const char SCROLL_HALF_UP[]           = "EmacsKeys.ScrollHalfUp";

} // namespace EmacsKeys
} // namespace Constants
