// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Core::Internal {

class LoggingViewer
{
public:
    static void showLoggingView();
    static void hideLoggingView();
};

} // namespace Core::Internal
