// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace CppEditor::Internal {
void registerSynchronizeMemberFunctionOrderQuickfix();
} // namespace CppEditor::Internal
