// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <utils/icon.h>

namespace DiffEditor {
namespace Icons {

const Utils::Icon TOP_BAR({
        {":/diffeditor/images/topbar.png", Utils::Theme::IconsBaseColor}});
const Utils::Icon UNIFIED_DIFF({
        {":/diffeditor/images/unifieddiff.png", Utils::Theme::IconsBaseColor}});
const Utils::Icon SIDEBYSIDE_DIFF({
        {":/diffeditor/images/sidebysidediff.png", Utils::Theme::IconsBaseColor}});

} // namespace Icons
} // namespace DiffEditor
