// Copyright (C) 2025 David M. Cotter
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Mcp::Constants {

const char ACTION_ID[] = "MCPServer.Action";
const char MENU_ID[] = "MCPServer.Menu";

// About action
const char ABOUT_ACTION_ID[] = "MCPServer.About";

} // namespace Mcp::Constants
