// Copyright (C) 2016 Jochen Becher
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include "qmt/controller/container.h"
#include "qmt/diagram/delement.h"

namespace qmt {

class QMT_EXPORT DContainer : public Container<DElement>
{
};

} // namespace qmt
