// Copyright (C) 2016 Lorenz Haas
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QtGlobal>

namespace Beautifier::Constants {

const char ACTION_ID[]              = "Beautifier.Action";
const char MENU_ID[]                = "Beautifier.Menu";
const char OPTION_CATEGORY[]        = "II.Beautifier";
const char OPTION_GENERAL_ID[]      = "aaa.General";
const char SETTINGS_DIRNAME[]       = "beautifier";
const char DOCUMENTATION_DIRNAME[]  = "documentation";
const char DOCUMENTATION_XMLROOT[]  = "beautifier_documentation";
const char DOCUMENTATION_XMLENTRY[] = "entry";
const char DOCUMENTATION_XMLKEYS[]  = "keys";
const char DOCUMENTATION_XMLKEY[]   = "key";
const char DOCUMENTATION_XMLDOC[]   = "doc";

} // Beautifier::Constants
